const path = require('path');

const plugins = [
  require.resolve('babel-plugin-transform-decorators-legacy').default,
  require.resolve('babel-plugin-transform-decorators'),
  require.resolve('babel-plugin-syntax-trailing-function-commas'),
  require.resolve('babel-plugin-transform-class-properties'),
  [
    require.resolve('babel-plugin-transform-object-rest-spread'),
    {
      useBuiltIns: true,
    },
  ],
  [
    require.resolve('babel-plugin-transform-react-jsx'),
    {
      useBuiltIns: true,
    },
  ],
  [
    require.resolve('babel-plugin-transform-runtime'),
    {
      helpers: false,
      polyfill: false,
      regenerator: true,
      moduleName: path.dirname(require.resolve('babel-runtime/package')),
    },
  ],
  require.resolve('babel-plugin-transform-flow-strip-types'),
  require.resolve('babel-plugin-transform-export-extensions'),
  [
    require.resolve('babel-plugin-transform-regenerator'),
    {
      async: false,
    },
  ],
];
let env = process.env.BABEL_ENV || process.env.NODE_ENV;
if (env === 'development' || env === 'test') {
  plugins.push.apply(plugins, [
    require.resolve('babel-plugin-transform-react-jsx-source'),
    require.resolve('babel-plugin-transform-react-jsx-self'),
  ]);
}

// Transform import statements to supported version
if (env === 'server' || env === 'test') {
  plugins.push.apply(plugins, [
    require.resolve('babel-plugin-dynamic-import-node'),
  ]);
} else {
  plugins.push.apply(plugins, [
    require.resolve('babel-plugin-dynamic-import-webpack'),
  ]);
}

module.exports = function(context, opts) {
  let config;
  require.resolve('babel-preset-latest');

  if (env === 'test') {
    config = {
      presets: [
        // ES features necessary for user's Node version
        [
          require('babel-preset-env').default,
          {
            targets: {
              node: 'current',
            },
            include: ['transform-es2015-destructuring'],
          },
        ],
        // JSX, Flow
        require.resolve('babel-preset-react'),
      ],
      plugins: plugins,
    };
  } else {
    config = {
      presets: [
        // Latest stable ECMAScript features
        ['latest', opts],
        // JSX, Flow
        require.resolve('babel-preset-react'),
        require.resolve('babel-preset-stage-2'),
      ],
      plugins: plugins.concat([
        // function* () { yield 42; yield 43; }
        [
          require.resolve('babel-plugin-transform-regenerator'),
          {
            // Async functions are converted to generators by babel-preset-latest
            async: false,
          },
        ],
      ]),
    };

    if (env === 'production') {
      plugins.push.apply(plugins, [
        require.resolve('babel-plugin-transform-flow-strip-types'),
        require.resolve('babel-plugin-transform-react-display-name'),
        require.resolve('babel-plugin-transform-react-constant-elements'),
        require.resolve('babel-plugin-transform-react-inline-elements')
        require.resolve('babel-plugin-transform-react-pure-class-to-function'),
      ]);
    }
  }

  return config;
};
